import { RcSdk } from "./RcSdk";

interface ITeamRemoveMemberBody {
    teamId: string;
    userId: string;
}

interface ITeamAddMemberBody {
    teamId: string;
    members: Array<{
        userId: string;
        roles: Array<string>;
    }>;
}

interface ITeamDeleteBody {
    teamId: string;
    roomsToRemove?: Array<string>;
}

interface ITeamCreateBody {
    name: string;
    type: 0 | 1;
    members?: Array<string>;
    room?: {
        readOnly: boolean;
    };
}

export class RcSdkTeam {
    private sdk: RcSdk;
    constructor(sdk: RcSdk) {
        this.sdk = sdk;
    }

    public async listAll(): Promise<any> {
        const response = await this.sdk.get(`teams.listAll`);
        return this.sdk.parseResponse(response);
    }

    public async members(teamId: string): Promise<any> {
        const response = await this.sdk.get(`teams.members?teamId=${teamId}`);
        return this.sdk.parseResponse(response);
    }

    public async info(teamId: string): Promise<any> {
        const response = await this.sdk.get(`teams.info?teamId=${teamId}`);
        return this.sdk.parseResponse(response);
    }

    public async delete(body: ITeamDeleteBody): Promise<any> {
        const response = await this.sdk.post(`teams.delete`, body);
        return this.sdk.parseResponse(response);
    }

    public async create(body: ITeamCreateBody): Promise<any> {
        const response = await this.sdk.post(`teams.create`, body);
        return this.sdk.parseResponse(response);
    }

    public async removeMember(body: ITeamRemoveMemberBody): Promise<any> {
        const response = await this.sdk.post(`teams.removeMember`, body);
        return this.sdk.parseResponse(response);
    }

    public async addMembers(body: ITeamAddMemberBody): Promise<any> {
        const response = await this.sdk.post(`teams.addMembers`, body);
        return this.sdk.parseResponse(response);
    }
}
